# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: GUI.pm,v 1.7 2003/08/17 17:26:19 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;
package GUI;

use POSIX;
use Locale::gettext;

#
# create the main object
#
sub new {
   my $that = shift;
   my $class = ref($that) || $that;

   my $self = {};
   $self->{'init'} = shift;

   bless($self, $class);

   $self->{'version'} = '0.5.0 (beta)';

   # initialize CA object
   $self->{'CA'} = CA->new($self->{'init'});

   # initialize OpenSSL object
   $self->{'OpenSSL'} = OpenSSL->new($self);

   # initialize CERT object
   $self->{'CERT'} = CERT->new();

   # initialize KEY object
   $self->{'KEY'} = KEY->new();

   # initialize REQ object
   $self->{'REQ'} = REQ->new();

   # initialize CONFIG object
   $self->{'TCONFIG'} = TCONFIG->new();

   # initialize fonts and styles
   my $font  = Gtk::Gdk::Font->fontset_load("-adobe-helvetica-bold-r-normal--*-120-*-*-*-*-*-*");
   $self->{'stylebold'} = Gtk::Style->new();
   $self->{'stylebold'}->font($font);

   $self->{'stylered'} = Gtk::Style->new();
   $self->{'stylered'}->fg('normal', Gtk::Gdk::Color->parse_color('red'));

   $self->{'stylegreen'} = Gtk::Style->new();
   $self->{'stylegreen'}->fg('normal', Gtk::Gdk::Color->parse_color('green'));

   # initialize main window
   $self->{'mw'} = Gnome::App->new("TinyCA", 
         "Tiny CA Management $self->{'version'}");

   $self->{'mw'}->set_policy(0, 1, 0);
   $self->{'mw'}->set_default_size(740, 570);
   $self->{'mw'}->signal_connect( 'delete_event', sub { Gtk->exit(0) });
   
   $self->create_nb();
   $self->{'mw'}->set_contents($self->{'nb'});

   $self->create_menu();

   $self->create_toolbar('startup');

   $self->create_bar();

   $self;
}

#
# create/update the main frame with the notebooks
#
sub create_mframe {
   my $self = shift;

   my($parsed, $calabel, $caframe, $rows, $table, @fields, $text, @childs,
         $label, $cert_export, $cert_revoke, $cert_delete, $certlabel,
         $certlistwin, @certtitles, @keytitles, $keylabel, $keylistwin,
         $reqlistwin, @reqtitles, $reqlabel);

   return if not defined($self->{'CA'}->{'actca'});
   return if $self->{'CA'}->{'actca'} eq "";
   my $ca = $self->{'CA'}->{'actca'};

   $parsed = $self->{'CERT'}->parse_cert( $self, 'CA');

   defined($parsed) || $self->print_error(
         gettext("Can't read CA certificate")
         );

   ### notebooktab for ca information
   if(not defined($self->{'cabox'})) {
      $self->{'cabox'} = Gtk::VBox->new(0, 0);
      $calabel = Gtk::Label->new(gettext("CA"));
      $self->{'nb'}->insert_page($self->{'cabox'}, $calabel, 0);
   } else {
      $self->{'nb'}->hide();
      $self->{'nb'}->remove_page(0);
      $self->{'cabox'}->destroy();
      $self->{'cabox'} = Gtk::VBox->new(0, 0);
      $calabel = Gtk::Label->new(gettext("CA"));
      $self->{'nb'}->insert_page($self->{'cabox'}, $calabel, 0);
   }

   # frame for CA informations
   $self->{'caframe'} = Gtk::Frame->new(gettext("CA Information: ").$ca);
   $self->{'caframe'}->border_width(5);
   $self->{'cabox'}->pack_start($self->{'caframe'}, 1, 1, 0);

   $self->{'catextbox'} = Gtk::VBox->new(0, 0);
   $self->{'caframe'}->add($self->{'catextbox'});

   # fingerprint in the top of caframe
   if(defined($self->{'cafingerprint'})) {
      $self->{'cafingerprint'}->destroy();
   }
   $self->{'cafingerprint'} = Gtk::Label->new(
         gettext("Fingerprint:")." ".$parsed->{'FINGERPRINT'});
   $self->{'catextbox'}->pack_start($self->{'cafingerprint'}, 0, 0, 0);

   # hbox in the bottom
   if(defined($self->{'cabottombox'})) {
      $self->{'cabottombox'}->destroy();
   }
   $self->{'cabottombox'} = Gtk::HBox->new(1, 0);
   $self->{'catextbox'}->pack_start($self->{'cabottombox'}, 0, 0, 0);

   # vbox in the bottom/left
   if(defined($self->{'caleftbox'})) {
      $self->{'caleftbox'}->destroy();
   }
   $self->{'caleftbox'} = Gtk::VBox->new(0, 0);
   $self->{'cabottombox'}->add($self->{'caleftbox'});

   # vbox in the bottom/right
   if(defined($self->{'carightbox'})) {
      $self->{'carightbox'}->destroy();
   }
   $self->{'carightbox'} = Gtk::VBox->new(0, 0);
   $self->{'cabottombox'}->add($self->{'carightbox'});

   # now fill the left vbox
   $rows  = 1;
   $table = Gtk::Table->new($rows, 2, 1);
   $self->{'caleftbox'}->add($table);

   if(defined($parsed->{'CN'})){
      $label = Gtk::Label->new(gettext("Common Name:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'CN'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'EMAIL'})){
      $label = Gtk::Label->new(gettext("eMail Address:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'EMAIL'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'O'})){
      $label = Gtk::Label->new(gettext("Organization:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'O'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'OU'})){
      foreach(@{$parsed->{'OU'}}) {
         $label = Gtk::Label->new(gettext("Organizational Unit:"));
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $label->set_padding(20, 0);
         $table->attach_defaults($label, 0, 1, $rows-1, $rows);

         $label = Gtk::Label->new($_);
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $label->set_padding(3, 0);
         $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
         $rows++;
         $table->resize($rows, 2);
      }
   }

   if(defined($parsed->{'L'})){
      $label = Gtk::Label->new(gettext("Location:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'L'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'ST'})){
      $label = Gtk::Label->new(gettext("State:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'ST'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'C'})){
      $label = Gtk::Label->new(gettext("Country:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(20, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'C'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   # now fill the right vbox
   $rows  = 1;
   $table = Gtk::Table->new($rows, 2, 1);
   $self->{'carightbox'}->add($table);

   if(defined($parsed->{'NOTBEFORE'})){
      $label = Gtk::Label->new(gettext("Creation Date:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'NOTBEFORE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'NOTAFTER'})){
      $label = Gtk::Label->new(gettext("Expiration Date:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'NOTAFTER'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }
     
   if(defined($parsed->{'KEYSIZE'})){
      $label = Gtk::Label->new(gettext("Keylength:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'KEYSIZE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'SIG_ALGORITHM'})){
      $label = Gtk::Label->new(gettext("Signature Algorithm:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'SIG_ALGORITHM'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   $self->{'catextbox'}->show_all();
   $self->{'caframe'}->show_all();

   ### notebooktab for certificates (split info and buttons)
   @certtitles = (gettext("Common Name"),
         gettext("eMail Address"),
         gettext("Organizational Unit"),
         gettext("Organization"),
         gettext("Location"),
         gettext("State"),
         gettext("Country"), 
         gettext("Status"),
         "index");

   if(not defined($self->{'certbox'})) {
      $self->{'certbox'} = Gtk::VBox->new(0, 0);
      $certlabel = Gtk::Label->new(gettext("Certificates"));
      $self->{'nb'}->insert_page($self->{'certbox'}, $certlabel, 1);

      # pane for list (top) and cert infos (bottom)
      $self->{'certpane'} = Gtk::VPaned->new();
      $self->{'certpane'}->set_handle_size(10);
      $self->{'certpane'}->set_gutter_size(8);
      $self->{'certpane'}->set_position(290);
      $self->{'certbox'}->add($self->{'certpane'});
   
      # now the list
      $certlistwin = Gtk::ScrolledWindow->new(undef, undef);
      $certlistwin->set_policy('automatic', 'automatic');
      $self->{'certpane'}->add1($certlistwin);
      
      $self->{'certlist'} = Gtk::CList->new_with_titles(@certtitles);

      for(my $i = 0; $i < 7; $i++) {
         $self->{'certlist'}->set_column_auto_resize ($i, 1);
      }
      $self->{'certlist'}->signal_connect('select_row', 
            \&_fill_cert_info, $self );
      $self->{'certlist'}->signal_connect('button_release_event', 
            \&_show_cert_menu, $self);
      $self->{'certlist'}->set_column_visibility(8, 0);
      $certlistwin->add($self->{'certlist'});

      # add a frame for the information
      $self->{'certframe'} = Gtk::Frame->new(gettext("Certificate Information"));
      $self->{'certframe'}->border_width(5);
      $self->{'certframe'}->set_usize(0,420);
      $self->{'certpane'}->add2($self->{'certframe'});

      

      # and the info page
      if(defined($self->{'certtextbox'})) {
         $self->{'certtextbox'}->destroy();
      }
      $self->{'certtextbox'} = Gtk::VBox->new( 0, 0 );
      $self->{'certframe'}->add($self->{'certtextbox'});

      # create popup menu
      _create_cert_menu($self);

   } else {
      $self->{'certlist'}->clear();
      if(defined($self->{'certtextbox'})) {
         $self->{'certtextbox'}->destroy();
      }
      $self->{'certtextbox'} = Gtk::VBox->new( 0, 0 );
      $self->{'certframe'}->add($self->{'certtextbox'});
   }

   $self->{'CERT'}->read_certlist($self);

   my $ind = 0;
   foreach my $n (@{$self->{'CERT'}->{'certlist'}}) {
      my ($name, $state) = split(/\%/, $n);
      my @line = split(/\:/, $name);
      my $row = $self->{'certlist'}->append(@line);
      $self->{'certlist'}->set_text($row, 7, $state);
      if($state eq gettext("VALID")) {
         $self->{'certlist'}->set_cell_style($row, 7, $self->{'stylegreen'});
      } else {
         $self->{'certlist'}->set_cell_style($row, 7, $self->{'stylered'});
      }
      $self->{'certlist'}->set_text($row, 8, $ind);
      $ind++;
   }
   # now select the first row to display certificate informations
   $self->{'certlist'}->select_row(0, 0);

   ### notebooktab for keys (split info and buttons)
   @keytitles = (gettext("Common Name"),
         gettext("eMail Address"),
         gettext("Organizational Unit"),
         gettext("Organization"),
         gettext("Location"),
         gettext("State"),
         gettext("Country"), 
         "index");

   if(not defined($self->{'keybox'})) {
      $self->{'keybox'} = Gtk::VBox->new(0, 0);
      $keylabel = Gtk::Label->new(gettext("Keys"));
      $self->{'nb'}->insert_page($self->{'keybox'}, $keylabel, 2);
   
      # now the list
      $keylistwin = Gtk::ScrolledWindow->new(undef, undef);
      $keylistwin->set_policy('automatic', 'automatic');
      $self->{'keybox'}->add($keylistwin);
      
      $self->{'keylist'} = Gtk::CList->new_with_titles(@keytitles);
      for(my $i = 0; $i < 6; $i++) {
         $self->{'keylist'}->set_column_auto_resize ($i, 1);
      }
      $self->{'keylist'}->set_column_visibility(7, 0);
      $keylistwin->add($self->{'keylist'});
      $self->{'keylist'}->signal_connect('button_release_event', 
            \&_show_key_menu, $self);

      # create popup menu
      _create_key_menu($self);

   } else {
      $self->{'keylist'}->clear();
   }

   $self->{'KEY'}->read_keylist($self);

   $ind = 0;
   foreach my $n (@{$self->{'KEY'}->{'keylist'}}) {
      my @line = split(/\:/, $n);
      my $row = $self->{'keylist'}->append(@line);
      $self->{'keylist'}->set_text($row, 7, $ind);
      $ind++;
   }
   # now select the first row
   $self->{'keylist'}->select_row(0, 0);

   ### notebooktab for requests (split info and buttons)
   @reqtitles = (gettext("Common Name"),
         gettext("eMail Address"),
         gettext("Organizational Unit"),
         gettext("Organization"),
         gettext("Location"),
         gettext("State"),
         gettext("Country"), 
         "index");

   if(not defined($self->{'reqbox'})) {
      $self->{'reqbox'} = Gtk::VBox->new(0, 0);
      $reqlabel = Gtk::Label->new(gettext("Requests"));
      $self->{'nb'}->insert_page($self->{'reqbox'}, $reqlabel, 3);

      # pane for list (top) and request infos (bottom)
      $self->{'reqpane'} = Gtk::VPaned->new();
      $self->{'reqpane'}->set_handle_size(10);
      $self->{'reqpane'}->set_gutter_size(8);
      $self->{'reqpane'}->set_position(310);
      $self->{'reqbox'}->add($self->{'reqpane'});
   
      # now the list
      $reqlistwin = Gtk::ScrolledWindow->new(undef, undef);
      $reqlistwin->set_policy('automatic', 'automatic');
      $self->{'reqpane'}->add1($reqlistwin);
      
      $self->{'reqlist'} = Gtk::CList->new_with_titles(@reqtitles);
      for(my $i = 0; $i < 6; $i++) {
         $self->{'reqlist'}->set_column_auto_resize ($i, 1);
      }
      $self->{'reqlist'}->signal_connect('select_row', 
            \&_fill_req_info, $self );
      $self->{'reqlist'}->signal_connect('button_release_event', 
            \&_show_req_menu, $self);
      $self->{'reqlist'}->set_column_visibility(7, 0);
      $reqlistwin->add($self->{'reqlist'});

      # add a frame for the information
      $self->{'reqframe'} = Gtk::Frame->new(gettext("Request Information"));
      $self->{'reqframe'}->border_width(5);
      $self->{'reqframe'}->set_usize(0, 400);
      $self->{'reqpane'}->add2($self->{'reqframe'});

      # and the info page
      if(defined($self->{'reqtextbox'})) {
         $self->{'reqtextbox'}->destroy();
      }
      $self->{'reqtextbox'} = Gtk::VBox->new( 0, 0 );
      $self->{'reqframe'}->add($self->{'reqtextbox'});

      # create popup menu
      _create_req_menu($self);

   } else {
      $self->{'reqlist'}->clear();
      if(defined($self->{'reqtextbox'})) {
         $self->{'reqtextbox'}->destroy();
      }
      $self->{'reqtextbox'} = Gtk::VBox->new( 0, 0 );
      $self->{'reqframe'}->add($self->{'reqtextbox'});
   }

   $self->{'REQ'}->read_reqlist($self);

   $ind = 0;
   foreach my $n (@{$self->{'REQ'}->{'reqlist'}}) {
      my ($name, $state) = split(/\%/, $n);
      my @line = split(/\:/, $name);
      my $row = $self->{'reqlist'}->append(@line);
      $self->{'reqlist'}->set_text($row, 7, $ind);
      $ind++;
   }
   # now select the first row to display certificate informations
   $self->{'reqlist'}->select_row(0, 0);

   $self->{'nb'}->show_all();
   $self->{'nb'}->signal_connect('switch_page', \&_act_toolbar, $self);

   return;
}

#
# create empty notebook, add to main window and configure
# 
sub create_nb {
   my $self = shift;

   $self->{'nb'} = Gtk::Notebook->new();
   $self->{'nb'}->set_tab_pos('top');
   $self->{'nb'}->set_homogeneous_tabs(1);

   return;
}

#
# create the applicationbar
#
sub create_bar {
   my $self = shift;
   
   $self->{'bar'} = Gnome::AppBar->new(0,1,"user");
   $self->{'bar'}->set_status("   Watch out...");
   $self->{'mw'}->set_statusbar($self->{'bar'});

   return;
}

#
# keep toolbar in sync with notebook
#
sub _act_toolbar {
   my ($nb, $self, $page, $page_num) = @_;
   my $mode = 'startup';

   if ($page_num == 0) {
      $mode = 'ca';
   } elsif ($page_num == 1) {
      $mode = 'cert';
   } elsif ($page_num == 2) {
      $mode = 'key';
   } elsif ($page_num == 3) {
      $mode = 'req';
   }

   $self->create_toolbar($mode);
}

#
# create the toolbar
#
sub create_toolbar {
   my $self = shift;
   my $mode = shift;

   my ($icon, $mask, $iconw, $item, @children, $c);
   
   if(defined($self->{'toolbar'})) {
      @children = $self->{'toolbar'}->children();
      for(my $i = 4; $i < @children; $i++) {
         $c = $children[$i];
         $c->destroy();
      }
   } else {
      $self->{'toolbar'} = Gtk::Toolbar->new('horizontal', 'both');
      $self->{'mw'}->set_toolbar($self->{'toolbar'});

      ## Buttons for all toolbars
      # Exit 
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Quit', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);
   
      $item = $self->{'toolbar'}->append_item(
            gettext("Exit"),
            gettext("Exit TinyCA"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', sub { $self->_exit_clean() });
   
      # Open
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Open', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);
   
      $item = $self->{'toolbar'}->append_item(
            gettext("Open CA"),
            gettext("Open an existing CA"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CA'}->get_open_name($self)});
   
      # New
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('New', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);
   
      $item = $self->{'toolbar'}->append_item(
            gettext("New CA"),
            gettext("Create a new CA"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CA'}->get_ca_create($self)});
   
      # Delete
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Trash', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);
   
      $item = $self->{'toolbar'}->append_item(
            gettext("Delete CA"),
            gettext("Delete an existing CA"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CA'}->get_ca_delete($self)});
   
      $self->{'toolbar'}->append_space();
   }

   
   if($mode eq 'ca') {
      
      # Export CA
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Save', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Export CA"),
            gettext("Export CA Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CA'}->export_ca_cert($self)});

      # Export CRL
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Save', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Export CRL"),
            gettext("Export Certificate Revocation List"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CA'}->export_crl($self)});
   
   } elsif($mode eq 'cert') {
      
      # View certificate extensions
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Search', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Details"),
            gettext("Show X.509 Extensions of the selected Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->show_details('cert') });

      # Show Certificate
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Search', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("View"),
            gettext("View selected Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->show_text('cert') });
      
      # Export certificate
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Save', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Export"),
            gettext("Export selected Certificate to file"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CERT'}->get_export_cert($self) });

      # Revoke certificate
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Stop', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Revoke"),
            gettext("Revoke selected Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CERT'}->get_revoke_cert($self) });

      # Delete certificate
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Trash', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Delete"),
            gettext("Delete selected (revoked) Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'CERT'}->get_del_cert($self) });
      
   } elsif($mode eq 'key') {
      
      # Export key
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Save', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Export"),
            gettext("Export selected Key to file"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'KEY'}->get_export_key($self) });

      # Delete key
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Trash', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Delete"),
            gettext("Delete selected Key"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'KEY'}->get_del_key($self) });
      
   } elsif($mode eq 'req') {

      # Show Details
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Search', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Details"),
            gettext("Show Details of the selected Request"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->show_details('req') });

      # Show Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Search', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("View"),
            gettext("View selected Request"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->show_text('req') });
      
      # New Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('New', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("New"),
            gettext("Generate new Key and Certificate Request"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'REQ'}->get_req_create($self) });
      
      # Import Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Revert', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Import"),
            gettext("Import a Certificate Request"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'REQ'}->get_import_req($self) });

      # Sign Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Properties', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Server"),
            gettext("Sign Certificate Request/Create Server Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { my $opts = {};
                  $opts->{'type'} = 'server';
                  $self->{'REQ'}->get_sign_req($self, $opts) });

      # Sign Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Properties', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Client"),
            gettext("Sign Certificate Request/Create Client Certificate"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { my $opts = {};
                  $opts->{'type'} = 'client';
                  $self->{'REQ'}->get_sign_req($self, $opts); });

      # Delete Request
      ($icon, $mask) = Gnome::Stock->pixmap_gdk('Trash', 'GPixmap');
      $iconw = Gtk::Pixmap->new($icon, $mask);

      $item = $self->{'toolbar'}->append_item(
            gettext("Delete"),
            gettext("Delete selected Certificate Request"),
            'Private',
            $iconw);
      $item->signal_connect('clicked', 
            sub { $self->{'REQ'}->get_del_req($self) });
   }

   return;
}

#
# create the menubar
#
sub create_menu {
   my $self = shift;

    $self->{'mw'}->create_menus(
               { type    => 'subtree',
                 label   => gettext("CA"),
                 subtree => [
                    { type        => 'item',
                      label       => gettext("Open CA"),
                      pixmap_type => 'stock',
                      pixmap_info => 'Menu_Open',
                      callback    => sub { $self->{'CA'}->get_open_name($self) }
                    },
                    { type        => 'item',
                      label       => gettext("New CA"),
                      pixmap_type => 'stock',
                      pixmap_info => 'Menu_New',
                      callback    => sub { $self->{'CA'}->get_ca_create($self)}
                    },
                    { type        => 'item',
                      label       => gettext("Delete CA"),
                      pixmap_type => 'stock',
                      pixmap_info => 'Menu_Trash',
                      callback    => sub { $self->{'CA'}->get_ca_delete($self)}
                    },
                    { type => 'separator',
                    },
                    { type        => 'item',
                      label       => 'Exit',
                      pixmap_type => 'stock',
                      pixmap_info => 'Menu_Quit',
                      callback    => sub { $self->_exit_clean() }
                    },
                 ]
               },
               { type    => 'subtree',
                 label   => gettext("Preferences"),
                 subtree => [
                    { type  => 'item',
                      label => gettext("Experts Only!!")
                    },
                    { type => 'separator'
                    },
                    { type        => 'item',
                      label       => gettext("OpenSSL Configuration"),
                      pixmap_type => 'stock',
                      pixmap_info => 'Menu_Properties',
                      callback    => sub{ $self->{'TCONFIG'}->config_openssl($self) }
                    }
                 ]
               },
               { type    => 'subtree',
                 label   => gettext("Help"),
                 subtree => [
                    { type     => 'item', 
                      label    => gettext("Help"),
                      callback => sub{ $self->show_help() }
                    },
                    {type => 'item', 
                     label => gettext("About TinyCA"),
                     pixmap_type => 'stock',
                     pixmap_info => 'Menu_About',
                     callback => sub { $self->about() }
                    }
                 ]
               }
              );

   return;
}

#
# pop-up to display request/cert as TXT
#
sub show_text {
   my $self  = shift;
   my $mode  = shift;
   my $ca = $self->{'CA'}->{'actca'};

   my($parsed, $ind, $t, $box, $label, $text, $vscrollbar, $name, $button_ok,
         $status, $row, $scrolled);

   if($mode eq 'req') {
      $row = $self->{'reqlist'}->selection();
      $ind = $self->{'reqlist'}->get_text($row, 7);
   } elsif($mode eq 'cert') {
      $row = $self->{'certlist'}->selection();
      $ind = $self->{'certlist'}->get_text($row, 8);
   } else {
      $self->print_error(gettext("Invalid mode for show_text():")." ".$mode);
      return;
   }

   if((not defined $ind) && ($mode eq 'req')) { 
      $self->print_info(gettext("Please select a Request first"));
      return;
   }elsif((not defined $ind) && ($mode eq 'cert')) {
      $self->print_info(gettext("Please select a certificate first"));
      return;
   }

   if($mode eq 'req') {
      ($name, $status) = split(/\%/, $self->{'REQ'}->{'reqlist'}->[$ind]);
   }elsif($mode eq 'cert') {
      ($name, $status) = split(/\%/, $self->{'CERT'}->{'certlist'}->[$ind]);
   }

   $name = MIME::Base64::encode($name, '');

   if($mode eq 'req') {
      $parsed = $self->{'REQ'}->parse_req( $self, $name);
   } elsif($mode eq 'cert') {
      $parsed = $self->{'CERT'}->parse_cert( $self, $name);
   }

   defined($parsed) || $self->print_error(gettext("Can't read file"));

   $t = $mode eq 'req'?gettext("Request"):gettext("Certificate");

   $box = Gnome::Dialog->new($t);
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(0);
   $box->set_default_size(550, 440);

   $button_ok = Gnome::Stock->button('Button_Ok');
   $button_ok->can_default(1);

   $box->action_area->set_layout('end');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );

   $button_ok->grab_default();
   $button_ok->signal_connect('clicked', sub { $box->destroy() });

   $label = Gtk::Label->new($t);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->pack_start($label, 0, 0, 0);

   $scrolled = Gtk::ScrolledWindow->new(undef, undef);
   $scrolled->set_policy('automatic', 'automatic');
   $box->vbox->pack_start($scrolled, 1, 1, 0);

   $text = Gtk::Text->new();
   $text->set_editable(0);
   $text->set_word_wrap(0);
   $text->set_line_wrap(0);
   $text->insert(undef, undef, undef, $parsed->{'TEXT'});
   $scrolled->add($text);

   $box->show_all();
   return;
}

#
# show request/certificate informations and extensions
#
sub show_details {
   my $self = shift;
   my $mode = shift;
   my $ca   = $self->{'CA'}->{'actca'};

   my($name, $status, $parsed, $key, $val, $row, $ind, $label, $table, 
         $rows, $box, $button_ok, $t);

   if($mode eq 'req') {
      $row = $self->{'reqlist'}->selection();
      $ind = $self->{'reqlist'}->get_text($row, 7);
   } elsif($mode eq 'cert') {
      $row = $self->{'certlist'}->selection();
      $ind = $self->{'certlist'}->get_text($row, 8);
   } else {
      $self->print_error(gettext("Invalid mode for show_details():")." ".$mode);
      return;
   }

   if((not defined $ind) && ($mode eq 'req')) { 
      $self->print_info(gettext("Please select a Request first"));
      return;
   }elsif((not defined $ind) && ($mode eq 'cert')) {
      $self->print_info(gettext("Please select a certificate first"));
      return;
   }

   if($mode eq 'req') {
      ($name, $status) = split(/\%/, $self->{'REQ'}->{'reqlist'}->[$ind]);
   }elsif($mode eq 'cert') {
      ($name, $status) = split(/\%/, $self->{'CERT'}->{'certlist'}->[$ind]);
   }

   $name = MIME::Base64::encode($name, '');

   if($mode eq 'req') {
      $parsed = $self->{'REQ'}->parse_req( $self, $name);
   } elsif($mode eq 'cert') {
      $parsed = $self->{'CERT'}->parse_cert( $self, $name);
   }

   defined($parsed) || $self->print_error(gettext("Can't read file"));

   $t = $mode eq 'req'?gettext("Request Details"):gettext("Certificate Details");

   $box = Gnome::Dialog->new($t);
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(0);

   $button_ok = Gnome::Stock->button('Button_Ok');
   $button_ok->can_default(1);

   $box->action_area->set_layout('end');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );

   $button_ok->grab_default();
   $button_ok->signal_connect('clicked', sub { $box->destroy() });

   $label = Gtk::Label->new($t);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   if(defined($parsed->{'CN'})){
      $label = Gtk::Label->new(gettext("Common Name:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'CN'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'EMAIL'})){
      $label = Gtk::Label->new(gettext("eMail Address:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'EMAIL'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'O'})){
      $label = Gtk::Label->new(gettext("Organization:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'O'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'OU'})){
      foreach(@{$parsed->{'OU'}}) {
         $label = Gtk::Label->new(gettext("Organizational Unit:"));
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $table->attach_defaults($label, 0, 1, $rows-1, $rows);

         $label = Gtk::Label->new($_);
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $label->set_padding(3, 0);
         $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
         $rows++;
         $table->resize($rows, 2);
      }
   }

   if(defined($parsed->{'L'})){
      $label = Gtk::Label->new(gettext("Location:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'L'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'ST'})){
      $label = Gtk::Label->new(gettext("State:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'ST'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'C'})){
      $label = Gtk::Label->new(gettext("Country:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'C'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'KEYSIZE'})){
      $label = Gtk::Label->new(gettext("Keylength:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'KEYSIZE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'SIG_ALGORITHM'})){
      $label = Gtk::Label->new(gettext("Signature Algorithm:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'SIG_ALGORITHM'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'TYPE'})){
      $label = Gtk::Label->new(gettext("Type:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'TYPE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(not keys(%{$parsed->{'EXT'}})) {
      return;
   }

   $t = $mode eq 'req'?gettext("Requested X.509 Extensions"):gettext("X.509v3 Extensions");
   $label = Gtk::Label->new($t);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   while(($key, $val) = each(%{$parsed->{'EXT'}})) {
      $label = Gtk::Label->new($key.":");
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($val->[0]);
      $label->set_justify('left');
      $label->set_line_wrap(1);
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
            
      if(@{$val} > 1) {
         for(my $i = 1; $val->[$i]; $i++) {
            $label = Gtk::Label->new($val->[$i]);
            $label->set_justify('left');
            $label->set_line_wrap(1);
            $label->set_alignment(0, 0);
            $label->set_padding(3, 0);
            $table->attach_defaults($label, 1, 2, $rows-1, $rows);

            $rows++;
            $table->resize($rows, 2);
         }
      }
   }
   $box->show_all();
}

#
# pop-up to verify request import
#
sub show_req_import_verification {
   my $self   = shift;
   my $opts   = shift;
   my $parsed = shift;

   my($box, $button_ok, $button_cancel, $label, $rows, $table, $t, $key,
         $val);

   $box = Gnome::Dialog->new(gettext("Import Request"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(0);

   $button_ok = Gnome::Stock->button('Button_Ok');
   $button_ok->can_default(1);
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );


   $button_ok->grab_default();
   $button_ok->signal_connect('clicked', 
         sub { $self->{'REQ'}->import_req($self, $opts, $parsed, $box) });
   $button_cancel->signal_connect('clicked', sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Import Request"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(
         gettext("Do you really want to import the following Certificate Request?"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   if(defined($parsed->{'CN'})){
      $label = Gtk::Label->new(gettext("Common Name:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'CN'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'EMAIL'})){
      $label = Gtk::Label->new(gettext("eMail Address:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'EMAIL'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'O'})){
      $label = Gtk::Label->new(gettext("Organization:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'O'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'OU'})){
      foreach(@{$parsed->{'OU'}}) {
         $label = Gtk::Label->new(gettext("Organizational Unit:"));
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $table->attach_defaults($label, 0, 1, $rows-1, $rows);

         $label = Gtk::Label->new($_);
         $label->set_justify('left');
         $label->set_alignment(0, 0);
         $label->set_padding(3, 0);
         $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
         $rows++;
         $table->resize($rows, 2);
      }
   }

   if(defined($parsed->{'L'})){
      $label = Gtk::Label->new(gettext("Location:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'L'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'ST'})){
      $label = Gtk::Label->new(gettext("State:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'ST'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'C'})){
      $label = Gtk::Label->new(gettext("Country:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'C'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'KEYSIZE'})){
      $label = Gtk::Label->new(gettext("Keylength:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'KEYSIZE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'SIG_ALGORITHM'})){
      $label = Gtk::Label->new(gettext("Signature Algorithm:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'SIG_ALGORITHM'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(defined($parsed->{'TYPE'})){
      $label = Gtk::Label->new(gettext("Type:"));
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($parsed->{'TYPE'});
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);
   
      $rows++;
      $table->resize($rows, 2);
   }

   if(not keys(%{$parsed->{'EXT'}})) {
      return;
   }

   $t = gettext("Requested X.509 Extensions");
   $label = Gtk::Label->new($t);
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   while(($key, $val) = each(%{$parsed->{'EXT'}})) {
      $label = Gtk::Label->new($key.":");
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $label = Gtk::Label->new($val->[0]);
      $label->set_justify('left');
      $label->set_line_wrap(1);
      $label->set_alignment(0, 0);
      $label->set_padding(3, 0);
      $table->attach_defaults($label, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
            
      if(@{$val} > 1) {
         for(my $i = 1; $val->[$i]; $i++) {
            $label = Gtk::Label->new($val->[$i]);
            $label->set_justify('left');
            $label->set_line_wrap(1);
            $label->set_alignment(0, 0);
            $label->set_padding(3, 0);
            $table->attach_defaults($label, 1, 2, $rows-1, $rows);

            $rows++;
            $table->resize($rows, 2);
         }
      }
   }
   $box->show_all();

   return;
}

#
# get name for open/delete a CA
#
sub show_select_ca_dialog {
   my $self   = shift;
   my $action = shift;
   my $opts   = shift;

   my ($box, $button_ok, $button_cancel, $label, $scrolled, $list, 
         $item, $name, $text);

   if($action eq 'open') {
      $text = gettext("Open CA");
   }elsif($action eq 'delete') {
      $text = gettext("Delete CA");
   }else {
      $self->print_error(gettext("Invalid action given: ").$action);
      return;
   }

   $box = Gnome::Dialog->new($text);
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_ok->can_default(1);
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread' );
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $label = Gtk::Label->new($text);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center' );
   $label->set_line_wrap(0 );
   $label->set_alignment(0.5, 0.5 );
   $box->vbox->add($label);

   $scrolled = Gtk::ScrolledWindow->new(undef, undef);
   $scrolled->set_policy('automatic', 'automatic' );
   $scrolled->border_width(0 );
   $scrolled->hscrollbar->set_update_policy('continuous' );
   $scrolled->vscrollbar->set_update_policy('continuous' );
   $box->vbox->add($scrolled);

   $list = Gtk::List->new();
   $scrolled->add_with_viewport($list);

   foreach(@{$self->{'CA'}->{'calist'}}) {
      next if (not defined $_ );
      $item = Gtk::ListItem->new($_);
      $item->{'name'} = $_;
      $list->append_items($item);
   }

   # activate doubleclick in the list
   $list->signal_connect('button_press_event', 
         sub { 
            if($_[1]->{'type'} eq '2button_press') {
               if(defined($list->selection)) {
                  $name = ($list->selection)[0]->{'name'};
                  if($action eq 'open') {
                     $opts->{'name'} = $name;
                     $self->{'CA'}->open_ca($self, $opts, $box);
                  }elsif($action eq 'delete') {
                     $self->{'CA'}->delete_ca($self, $name, $box);
                  }else {
                     $self->print_error(
                        gettext("Invalid action for show_select_ca_dialog(): ").$action);
                  }
               }
               return(1);
            }
            return(0);
         }
   );

   $button_ok->signal_connect('clicked', 
         sub { 
            if(defined($list->selection)) {
               $name = ($list->selection)[0]->{'name'};
               if($action eq 'open') {
                  $opts->{'name'} = $name;
                  $self->{'CA'}->open_ca($self, $opts, $box);
               }elsif($action eq 'delete') {
                  $self->{'CA'}->delete_ca($self, $name, $box);
               }else {
                  $self->print_error(
                     gettext("Invalid action for show_select_ca_dialog(): ").$action);
               }
            }
         }
   );
   $button_cancel->signal_connect('clicked', sub { $box->destroy() });
   $button_ok->grab_default();

   $box->show_all();
}

#
# get data for creating a new request
#
sub show_req_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $reqtable, $radiobox, $key1, $key2,
         $key3, $entry, $label);

   $box = Gnome::Dialog->new(gettext("Create Request"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread' );
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'REQ'}->get_req_create($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Create a new Certificate Request"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center' );
   $label->set_line_wrap(0 );
   $label->set_alignment(0.5, 0.5 );
   $box->vbox->add($label);

   # table for request data
   $reqtable = Gtk::Table->new(1, 13, 0);
   $reqtable->set_col_spacing(0, 10);
   $box->vbox->add($reqtable);

   $label = Gtk::Label->new(gettext("Common Name (eg, your Name,"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'CN'}) if(defined($opts->{'CN'}));
   $reqtable->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'CN'});
   $entry->set_visibility(1);
   $entry->grab_focus();

   $label = Gtk::Label->new(gettext("                   your eMail Address"));
   $label->set_justify('right');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 2, 3);

   $label = Gtk::Label->new(gettext("                   or the Servers Name)"));
   $label->set_justify('right');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 3, 4);

   $label = Gtk::Label->new(gettext("eMail Address:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 4, 5);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'EMAIL'}) if(defined($opts->{'EMAIL'}));
   $reqtable->attach_defaults($entry, 1, 2, 4, 5);
   $entry->signal_connect('changed', \&_entry_to_var, $entry,\$opts->{'EMAIL'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Password (protect your private Key):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 5, 6);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'passwd'}) if(defined($opts->{'passwd'}));
   $reqtable->attach_defaults($entry, 1, 2, 5, 6);
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'passwd'});
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("Password (confirmation):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 6, 7);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'passwd2'}) if(defined($opts->{'passwd2'}));
   $reqtable->attach_defaults($entry, 1, 2, 6, 7);
   $entry->signal_connect('changed',\&_entry_to_var,$entry,\$opts->{'passwd2'});
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("Country Name (2 letter code):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 7, 8);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'C'}) if(defined($opts->{'C'}));
   $reqtable->attach_defaults($entry, 1, 2, 7, 8);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'C'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("State or Province Name:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 8, 9);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'ST'}) if(defined($opts->{'ST'}));
   $reqtable->attach_defaults($entry, 1, 2, 8, 9);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'ST'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Locality Name (eg. city):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 9, 10);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'L'}) if(defined($opts->{'L'}));
   $reqtable->attach_defaults($entry, 1, 2, 9, 10);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'L'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Organization Name (eg. company):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 10, 11);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'O'}) if(defined($opts->{'O'}));
   $reqtable->attach_defaults($entry, 1, 2, 10, 11);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'O'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Organizational Unit Name (eg. section):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 11, 12);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'OU'}) if(defined($opts->{'OU'}));
   $reqtable->attach_defaults($entry, 1, 2, 11, 12);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'OU'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Keylength:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $reqtable->attach_defaults($label, 0, 1, 12, 13);

   $radiobox = Gtk::HBox->new(0, 0);
   $key1 = Gtk::RadioButton->new('1024');
   $key1->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '1024');
   $key1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 1024);
   $radiobox->add($key1);

   $key2 = Gtk::RadioButton->new('2048', $key1);
   $key2->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '2048');
   $key2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 2048);
   $radiobox->add($key2);

   $key3 = Gtk::RadioButton->new('4096', $key1);
   $key3->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '4096');
   $key3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 4096);
   $radiobox->add($key3);

   $reqtable->attach_defaults($radiobox, 1, 2, 12, 13);

   $box->show_all();

   return;
}

#
# get data for revoking a certificate
#
sub show_cert_revoke_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $table, $entry, $t);

   $t = gettext("Revoke Certificate");

   $box = Gnome::Dialog->new($t);
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6);
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'CERT'}->get_revoke_cert($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new($t);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for data
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("CA Password:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'passwd'});
   $entry->grab_focus();
   $entry->set_visibility(0);

   $box->show_all();
         
   return;
}

#
# get data for exporting a crl
#
sub show_crl_export_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $format1, $format2,
         $format3, $table, $entry, $fileentry);

   $box = Gnome::Dialog->new(gettext("Export CRL"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'CA'}->export_crl($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Export Revocation List to File"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for file selection
   $table = Gtk::Table->new(3, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("File:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $fileentry = Gnome::FileEntry->new('', gettext("Export CRL"));
   $fileentry->gnome_entry->set_max_saved(10);
   $fileentry->set_directory(0);
   $table->attach_defaults($fileentry, 1, 2, 0, 1);
   $fileentry->gnome_entry->entry->set_text($opts->{'outfile'})
      if(defined($opts->{'outfile'}));
   $fileentry->gnome_entry->entry->signal_connect(
         'changed', \&_entry_to_var,
         $fileentry->gnome_entry->entry,
         \$opts->{'outfile'});
   $fileentry->grab_focus();

   $label = Gtk::Label->new(gettext("CA Password:"));
   $table->attach_defaults($label, 0, 1, 1, 2);
   $entry = Gtk::Entry->new();
   $table->attach_defaults($entry, 1, 2, 1, 2);
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'passwd'});
   $entry->grab_focus();
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("Valid for (Days):"));
   $table->attach_defaults($label, 0, 1, 2, 3);
   $entry = Gtk::Entry->new();
   $table->attach_defaults($entry, 1, 2, 2, 3);
   $entry->set_text($opts->{'days'}) if(defined($opts->{'days'}));
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'days'});
   $entry->grab_focus();
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Export Format:"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $format1 = Gtk::RadioButton->new(gettext("PEM"));
   $format1->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'PEM');
   $format1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'PEM', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format1);

   $format2 = Gtk::RadioButton->new(
         gettext("DER"), $format1);
   $format2->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'DER');
   $format2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'DER', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format2);

   $format3 = Gtk::RadioButton->new(
         gettext("TXT"), $format1);
   $format3->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'TXT');
   $format3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'TXT', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format3);

   $box->show_all();

   return;
}

#
# get data for exporting a ca certificate
#
sub show_ca_export_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $format1, $format2,
         $format3, $table, $entry, $fileentry);

   $box = Gnome::Dialog->new(gettext("Export CA Certificate"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'CA'}->export_ca_cert($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Export CA Certificate to File"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for file selection
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("File:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $fileentry = Gnome::FileEntry->new('', gettext("Export CA Certificate"));
   $fileentry->gnome_entry->set_max_saved(10);
   $fileentry->set_directory(0);
   $table->attach_defaults($fileentry, 1, 2, 0, 1);
   $fileentry->gnome_entry->entry->set_text($opts->{'outfile'})
      if(defined($opts->{'outfile'}));
   $fileentry->gnome_entry->entry->signal_connect(
         'changed', \&_entry_to_var,
         $fileentry->gnome_entry->entry,
         \$opts->{'outfile'});
   $fileentry->grab_focus();

   $label = Gtk::Label->new(gettext("Export Format:"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $format1 = Gtk::RadioButton->new(gettext("PEM"));
   $format1->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'PEM');
   $format1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'PEM', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format1);

   $format2 = Gtk::RadioButton->new(
         gettext("DER"), $format1);
   $format2->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'DER');
   $format2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'DER', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format2);

   $format3 = Gtk::RadioButton->new(
         gettext("TXT"), $format1);
   $format3->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'TXT');
   $format3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'TXT', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format3);

   $box->show_all();

   return;
}

#
# get data for exporting a key
#
sub show_key_export_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $table, $entry, $fileentry,
         $format1, $format2, $format3, $format4, $passbox, $pass1, $pass2);

   $box = Gnome::Dialog->new(gettext("Export Key"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'KEY'}->get_export_key($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Export Key to File"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for file selection
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("File:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $fileentry = Gnome::FileEntry->new('', gettext("Export Key"));
   $fileentry->gnome_entry->set_max_saved(10);
   $fileentry->set_directory(0);
   $table->attach_defaults($fileentry, 1, 2, 0, 1);
   $fileentry->gnome_entry->entry->set_text($opts->{'outfile'})
      if(defined($opts->{'outfile'}));
   $fileentry->gnome_entry->entry->signal_connect(
         'changed', \&_entry_to_var,
         $fileentry->gnome_entry->entry,
         \$opts->{'outfile'});
   $fileentry->grab_focus();

   $label = Gtk::Label->new(gettext("Export Format:"));
   $label->set_justify('left');
   $label->set_line_wrap(0);
   $label->set_alignment(0, 0);
   $box->vbox->add($label);

   $format1 = Gtk::RadioButton->new(gettext("PEM (Key)"));
   $format1->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'PEM');
   $format1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'PEM', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format1);

   $format2 = Gtk::RadioButton->new(
         gettext("DER (Key without Passphrase)"), $format1);
   $format2->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'DER');
   $format2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'DER', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format2);

   $format3 = Gtk::RadioButton->new(
         gettext("PKCS#12 (Certificate & Key)"), $format1);
   $format3->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'P12');
   $format3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'P12', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format3);

   if(-x $self->{'init'}->{'zipbin'}) {
      $format4 = Gtk::RadioButton->new(
            gettext("Zip (Certificate & Key)"), $format1);
      $format4->set_active(1)
         if(defined($opts->{'format'}) && $opts->{'format'} eq 'ZIP');
      $format4->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
            'ZIP', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
      $box->vbox->add($format4);
   }

   $label = Gtk::Label->new(gettext("Without Passphrase (PEM)"));
   $label->set_justify('left');
   $label->set_line_wrap(0);
   $label->set_alignment(0, 0);
   $box->vbox->add($label);

   $passbox = Gtk::HBox->new(0, 0);
   $box->vbox->add($passbox);

   $pass1 = Gtk::RadioButton->new(gettext("Yes"));
   $pass1->set_active(1)
      if(defined($opts->{'nopass'}) && $opts->{'nopass'} == 1);
   $pass1->signal_connect('toggled', 
         \&_toggle_to_var, \$opts->{'nopass'}, 1);
   $passbox->add($pass1);

   $pass2 = Gtk::RadioButton->new(gettext("No"), $pass1);
   $pass2->set_active(1)
      if(defined($opts->{'nopass'}) && $opts->{'nopass'} == 0);
   $pass2->signal_connect('toggled', 
         \&_toggle_to_var, \$opts->{'nopass'}, 0);
   $passbox->add($pass2);
   

   $box->show_all();

   return;
}

#
# get password for exporting keys
#
sub show_key_nopasswd_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $table, $entry, $t);

   $t = gettext("Export Key without Passphrase");
   
   $box = Gnome::Dialog->new($t);
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6);
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'KEY'}->get_export_key($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new($t);
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(gettext("I hope you know what you\'re doing?"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(
         gettext("The Key Passphrase is needed for decryption of the Key"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for data
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("Password:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'passwd'});
   $entry->grab_focus();
   $entry->set_visibility(0);

   $box->show_all();
         
   return;
}

#
# get filename for importing a request
#
sub show_req_import_dialog {
   my $self = shift;

   my $opts = {};
   my($box, $button_ok, $button_cancel, $entry, $table, $label);

   $box = Gnome::Dialog->new(gettext("Import Request"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'REQ'}->get_import_req($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Import Request from File"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for data
   $table = Gtk::Table->new(2, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("File:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gnome::FileEntry->new('', gettext("Import Request File"));
   $entry->gnome_entry->set_max_saved(10);
   $entry->set_directory(0);
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->gnome_entry->entry->signal_connect(
         'changed', \&_entry_to_var,
         $entry->gnome_entry->entry,
         \$opts->{'infile'});
   $entry->grab_focus();

   $box->show_all();

   return;
}

#
# get data for exporting a certificate
#
# FIXME merge function with show_key_export_dialog()
sub show_cert_export_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $table, $entry, $fileentry,
         $format1, $format2, $format3, $format4, $format5);

   $box = Gnome::Dialog->new(gettext("Export Certificate"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'CERT'}->get_export_cert($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Export Certificate to File"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for file selection
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("File:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $fileentry = Gnome::FileEntry->new('', gettext("Export Certificate"));
   $fileentry->gnome_entry->set_max_saved(10);
   $fileentry->set_directory(0);
   $table->attach_defaults($fileentry, 1, 2, 0, 1);
   $fileentry->gnome_entry->entry->set_text($opts->{'outfile'})
      if(defined($opts->{'outfile'}));
   $fileentry->gnome_entry->entry->signal_connect(
         'changed', \&_entry_to_var,
         $fileentry->gnome_entry->entry,
         \$opts->{'outfile'});
   $fileentry->grab_focus();

   $label = Gtk::Label->new(gettext("Export Format:"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $format1 = Gtk::RadioButton->new(gettext("PEM (Certificate)"));
   $format1->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'PEM');
   $format1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'PEM', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format1);

   $format2 = Gtk::RadioButton->new(
         gettext("DER (Certificate)"), $format1);
   $format2->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'DER');
   $format2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'DER', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format2);

   $format3 = Gtk::RadioButton->new(
         gettext("PKCS#12 (Certificate & Key)"), $format1);
   $format3->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'P12');
   $format3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'P12', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format3);

   if(-x $self->{'init'}->{'zipbin'}) {
      $format4 = Gtk::RadioButton->new(
            gettext("Zip (Certificate & Key)"), $format1);
      $format4->set_active(1)
         if(defined($opts->{'format'}) && $opts->{'format'} eq 'ZIP');
      $format4->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
            'ZIP', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
      $box->vbox->add($format4);
   }

   $format5 = Gtk::RadioButton->new(
         gettext("TXT (Certificate)"), $format1);
   $format5->set_active(1)
      if(defined($opts->{'format'}) && $opts->{'format'} eq 'TXT');
   $format5->signal_connect('toggled', \&_toggle_to_var, \$opts->{'format'}, 
         'TXT', \$opts->{'outfile'}, \$opts->{'format'}, $fileentry);
   $box->vbox->add($format5);

   $box->show_all();

   return;
}

#
# get export passwd for pkcs#12
#
sub show_p12_export_dialog {
   my $self = shift;
   my $opts = shift;
   my $mode = shift;

   my ($box, $label,$table, $entry, $button_ok, $button_cancel, $radiobox,
         $includeca1, $includeca2);

   $box = Gnome::Dialog->new(gettext("Export Key to PKCS#12"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   if($mode eq 'key') {
      $button_ok->signal_connect('clicked', 
         sub { $self->{'KEY'}->get_export_key($self, $opts, $box) });
   } elsif($mode eq 'cert') {
      $button_ok->signal_connect('clicked', 
         sub { $self->{'CERT'}->get_export_cert($self, $opts, $box) });
   }
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Export Key to PKCS#12"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for storage name
   $table = Gtk::Table->new(2, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("Key Password:"));
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'passwd'}) if(defined($opts->{'passwd'}));
   $entry->set_visibility(0);
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', 
         \&_entry_to_var, $entry, \$opts->{'passwd'});
   $entry->grab_focus();

   $label = Gtk::Label->new(gettext("Export Password:"));
   $table->attach_defaults($label, 0, 1, 1, 2);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'p12passwd'}) if(defined($opts->{'p12passwd'}));
   $entry->set_visibility(0);
   $table->attach_defaults($entry, 1, 2, 1, 2);
   $entry->signal_connect('changed', 
         \&_entry_to_var, $entry, \$opts->{'p12passwd'});

   $label = Gtk::Label->new(gettext("Add CA Certificate tp PKCS#12 structure"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $radiobox = Gtk::HBox->new(0, 0);
   $box->vbox->add($radiobox);

   $includeca1 = Gtk::RadioButton->new(gettext("Yes"));
   $includeca1->set_active(1) 
      if(defined($opts->{'includeca'}) && $opts->{'includeca'} == 1);
   $includeca1->signal_connect('toggled', 
         \&_toggle_to_var, \$opts->{'includeca'}, 1);
   $radiobox->add($includeca1);

   $includeca2 = Gtk::RadioButton->new(gettext("No"), $includeca1);
   $includeca2->set_active(1) 
      if(defined($opts->{'includeca'}) && $opts->{'includeca'} == 0);
   $includeca2->signal_connect('toggled', 
         \&_toggle_to_var, \$opts->{'includeca'}, 1);
   $radiobox->add($includeca2);

   $box->show_all();

   return;
}

#
# get data for signing a request
#
sub show_req_sign_dialog {
   my $self      = shift;
   my $opts    = shift;

   my($box, $button_ok, $button_cancel, $entry, $table, $label);

   $box = Gnome::Dialog->new(gettext("Sign Request"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'REQ'}->get_sign_req($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Sign Request"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for data
   $table = Gtk::Table->new(2, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("CA Password:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var,$entry,\$opts->{'passwd'});
   $entry->grab_focus();
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("Valid for (Days):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $table->attach_defaults($label, 0, 1, 1, 2);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'days'}) if(defined($opts->{'days'}));
   $table->attach_defaults($entry, 1, 2, 1, 2);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'days'});

   $box->show_all();

   return;
}

#
# get data for creating a new CA
#
sub show_ca_dialog {
   my $self = shift;
   my $opts = shift;

   my ($box, $button_ok, $button_cancel, $label, $table, $entry, 
         $catable, $radiobox, $key1, $key2, $key3);

   $box = Gnome::Dialog->new(gettext("Create New CA"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { $self->{'CA'}->get_ca_create($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Create a new CA"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   # small table for storage name
   $table = Gtk::Table->new(1, 2, 0);
   $table->set_col_spacing(0, 10);
   $box->vbox->add($table);

   $label = Gtk::Label->new(gettext("Name (for local storage):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $table->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'name'}) if(defined($opts->{'name'}));
   $table->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'name'});
   $entry->grab_focus();

   $label = Gtk::Label->new(gettext("Data for CA Certificate"));
   $box->vbox->add($label);

   # table for ca data
   $catable = Gtk::Table->new(1, 13, 0);
   $catable->set_col_spacing(0, 10);
   $box->vbox->add($catable);

   $label = Gtk::Label->new(gettext("Common Name (for the CA):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 0, 1);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'CN'}) if(defined($opts->{'CN'}));
   $catable->attach_defaults($entry, 1, 2, 0, 1);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'CN'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Country Name (2 letter code):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 1, 2);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'C'}) if(defined($opts->{'C'}));
   $catable->attach_defaults($entry, 1, 2, 1, 2);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'C'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Password (needed for signing):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 2, 3);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'passwd'}) if(defined($opts->{'passwd'}));
   $catable->attach_defaults($entry, 1, 2, 2, 3);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'passwd'});
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("Password (confirmation):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 3, 4);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'passwd2'}) if(defined($opts->{'passwd2'}));
   $catable->attach_defaults($entry, 1, 2, 3, 4);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'passwd2'});
   $entry->set_visibility(0);

   $label = Gtk::Label->new(gettext("State or Province Name:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 4, 5);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'ST'}) if(defined($opts->{'ST'}));
   $catable->attach_defaults($entry, 1, 2, 4, 5);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'ST'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Locality Name (eg. city):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 5, 6);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'L'}) if(defined($opts->{'L'}));
   $catable->attach_defaults($entry, 1, 2, 5, 6);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'L'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Organization Name (eg. company):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 6, 7);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'O'}) if(defined($opts->{'O'}));
   $catable->attach_defaults($entry, 1, 2, 6, 7);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'O'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Organizational Unit Name (eg. section):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 7, 8);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'OU'}) if(defined($opts->{'OU'}));
   $catable->attach_defaults($entry, 1, 2, 7, 8);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'OU'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("eMail Address:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 8, 9);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'EMAIL'}) if(defined($opts->{'EMAIL'}));
   $catable->attach_defaults($entry, 1, 2, 8, 9);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'EMAIL'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Valid for (Days):"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 9, 10);
   $entry = Gtk::Entry->new();
   $entry->set_text($opts->{'days'}) if(defined($opts->{'days'}));
   $catable->attach_defaults($entry, 1, 2, 9, 10);
   $entry->signal_connect('changed', \&_entry_to_var, $entry, \$opts->{'days'});
   $entry->set_visibility(1);

   $label = Gtk::Label->new(gettext("Keylength:"));
   $label->set_justify('left');
   $label->set_alignment(0, 0);
   $catable->attach_defaults($label, 0, 1, 10, 11);

   $radiobox = Gtk::HBox->new(0, 0);
   $key1 = Gtk::RadioButton->new('1024');
   $key1->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '1024');
   $key1->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 1024);
   $radiobox->add($key1);

   $key2 = Gtk::RadioButton->new('2048', $key1);
   $key2->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '2048');
   $key2->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 2048);
   $radiobox->add($key2);

   $key3 = Gtk::RadioButton->new('4096', $key1);
   $key3->set_active(1) 
      if(defined($opts->{'bits'}) && $opts->{'bits'} == '4096');
   $key3->signal_connect('toggled', \&_toggle_to_var, \$opts->{'bits'}, 4096);
   $radiobox->add($key3);

   $catable->attach_defaults($radiobox, 1, 2, 10, 11);

   $box->show_all();

   return;
}

#
# subroutines for pop-up boxes
# 
sub show_help {
   my $self = shift;

   $self->print_info(gettext("You are kidding, are you??"));

   return;
}

#
#  About dialog
#
sub about {
   my $self = shift;

   my $aboutdialog = Gnome::About->new(
         'TinyCA', $self->{'version'}, 
         '(C) 2002 Stephan Martin',
         'Stephan Martin <sm@sm-zone.net>', 
         "http://tinyca.sm-zone.net\n\n".
         "This program is free software published under the GNU Public License");

   $aboutdialog->set_title(('About').' TinyCA');
   $aboutdialog->set_position('mouse' );
   $aboutdialog->set_policy(1, 1, 0);
   $aboutdialog->set_modal(1);
   $aboutdialog->realize();
   $aboutdialog->show();
   
   return;
}

#
#  Error message box, kills application
#
sub print_error {
   my $self = shift;
   my $t    = shift;
   
   my $box = Gnome::MessageBox->new($t, 'error');
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   my $button = Gnome::Stock->button('Button_Ok');
   $button->signal_connect('clicked', sub { $self->_exit_clean(1) });
   $button->can_default(1);

   my $actionarea = Gtk::HButtonBox->new();
   $box->vbox->add($actionarea);
   $actionarea->pack_start($button, 1, 1, 0);
   $actionarea->set_layout('end');
   $actionarea->set_spacing(8);
   $actionarea->set_child_ipadding(7, 0);

   $button->grab_default();

   $box->show_all();
}

#
#  Warning message box
#
sub print_warning {
   my $self = shift;
   my $t    = shift;
   
   my $box = Gnome::MessageBox->new($t, 'warning');
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   my $button = Gnome::Stock->button('Button_Ok');
   $button->signal_connect('clicked', sub { $box->destroy() });
   $button->can_default(1);

   my $actionarea = Gtk::HButtonBox->new();
   $box->vbox->add($actionarea);
   $actionarea->pack_start($button, 1, 1, 0);
   $actionarea->set_layout('end');
   $actionarea->set_spacing(8);
   $actionarea->set_child_ipadding(7, 0);

   $button->grab_default();

   $box->show_all();

   return;
}

#
#  Info message box
#
sub print_info {
   my $self = shift;
   my $t    = shift;
   
   my $box = Gnome::MessageBox->new($t, 'info');
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   my $button = Gnome::Stock->button('Button_Ok');
   $button->signal_connect('clicked', sub { $box->destroy() });
   $button->can_default(1);

   my $actionarea = Gtk::HButtonBox->new();
   $box->vbox->add($actionarea);
   $actionarea->pack_start($button, 1, 1, 0);
   $actionarea->set_layout('end');
   $actionarea->set_spacing(8);
   $actionarea->set_child_ipadding(7, 0);

   $button->grab_default();

   $box->show_all();

   return;
}

#
# get confirmation for deleting a request
#
sub show_del_confirm {
   my $self = shift;
   my $file = shift;
   my $type = shift;

   my $t = '';
   if($type eq 'req') {
      $t = gettext("Do you really want to delete the selected Request?");
   }elsif($type eq 'key') {
      $t = gettext("Do you really want to delete the selected Key?");
   }elsif($type eq 'cert') {
      $t = gettext("Do you really want to delete the selected Certificate?");
   }else{
      $self->print_error("Invalid type in show_del_confirm(): ".$type);
   }
      
   my $box = Gnome::MessageBox->new($t, 'question');
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   my $actionarea = Gtk::HButtonBox->new();
   $actionarea->set_layout('spread');
   $actionarea->set_spacing(6);
   $actionarea->set_child_ipadding(7, 0);
   $box->vbox->add($actionarea);

   my $button = Gnome::Stock->button('Button_Ok');
   if($type eq 'req') {
      $button->signal_connect('clicked', sub { 
           $self->{'REQ'}->del_req($self, $file);
           $box->destroy() });
   }elsif($type eq 'key') {
      $button->signal_connect('clicked', sub { 
           $self->{'KEY'}->del_key($self, $file);
           $box->destroy() });
   }elsif($type eq 'cert') {
      $button->signal_connect('clicked', sub {
            $self->{'CERT'}->del_cert($self, $file);
            $box->destroy() });
   }

   $actionarea->pack_start($button, 1, 1, 0);
   $button->can_default(1);
   $button->grab_default();

   $button = Gnome::Stock->button('Button_Cancel');
   $button->signal_connect('clicked', sub { $box->destroy(); return });
   $actionarea->pack_start($button, 1, 1, 0);

   $box->show_all();
}

#
# show warning - overwrite key
#
sub show_req_overwrite_warning {
   my $self = shift;
   my $opts = shift;

   my ($box, $actionarea, $button, $t);

   $t = gettext("The Key or the Request is already existing!");
   $t .= "\n\n";
   $t .= gettext("If the corresponding certificate it\'s not expired or revoked ");
   $t .= gettext("you won\'t be able to sign this request!");

   $box = Gnome::MessageBox->new($t, 'warning');
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   $actionarea = Gtk::HButtonBox->new();
   $box->vbox->add($actionarea);
   $actionarea->set_layout('end');
   $actionarea->set_spacing(8);
   $actionarea->set_child_ipadding(7, 0);

   $button = Gnome::Stock->button('Button_Ok');
   $button->signal_connect('clicked', 
         sub { $self->{'REQ'}->create_req($self, $opts); $box->destroy() });
   $actionarea->pack_start($button, 1, 1, 0);

   $button = Gnome::Stock->button('Button_Cancel');
   $button->signal_connect('clicked', sub { 
         $box->destroy() });
   $button->can_default(1);
   $actionarea->pack_start($button, 1, 1, 0);
   $button->grab_default();

   $box->show_all();

   return;
}

#
# show warning - certificate expiration date
#
sub show_req_date_warning {
   my $self = shift;
   my $opts = shift;

   my ($box, $label, $button_ok, $button_cancel, $t);

   $t = gettext("The Certificate will be longer valid than your CA!");
   $t .= "\n";
   $t .= gettext("This may cause problems with some software!!");

   $box = Gnome::Dialog->new(gettext("Expirationdate Warning"));
   $box->close_hides(0);
   $box->set_close(1);
   $box->set_position('mouse' );
   $box->set_policy(0, 0, 0);
   $box->set_modal(1);
   $box->realize();

   $label = Gtk::Label->new($t);
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6);
   $box->action_area->set_child_ipadding(7, 0);

   $button_ok = Gnome::Stock->button('Button_Ok');
   $button_ok->signal_connect('clicked', 
         sub { $opts->{'ignoredate'} = 'true';
               $self->{'REQ'}->get_sign_req($self, $opts, $box); });
   $box->action_area->pack_start($button_ok, 1, 1, 0);

   $button_cancel = Gnome::Stock->button('Button_Cancel');
   $button_cancel->signal_connect('clicked', sub { 
         $self->show_req_sign_dialog($opts);
         $box->destroy();
         });
   $button_cancel->can_default(1);
   $box->action_area->pack_start($button_cancel, 1, 1, 0);
   $button_cancel->grab_default();

   $box->show_all();

}

#
# get confirmation for overwriting certificate
#
sub show_cert_overwrite_confirm {
   my $self = shift;
   my $opts = shift;

   my($box, $button_ok, $button_cancel, $label);
   $box = Gnome::Dialog->new(gettext("Overwrite Certificate"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );
   $button_ok->can_default(1);
   $button_ok->grab_default();

   $button_ok->signal_connect('clicked', 
         sub { $opts->{'overwrite'} = 'true';
               $self->{'REQ'}->get_sign_req($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("Overwrite Certificate"));
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(gettext("There seems to be an certificate already."));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(gettext("Creating a new one (overwrite) will fail if it\'s not revoked or expired!"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $label = Gtk::Label->new(gettext("Really try to overwrite the Certificate?"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);

   $box->show_all();
   return;
}

#
# main screen for configuration
#
sub show_configbox {
   my $self = shift;
   my $name = shift;

   my ($box, $nb, $vbox, $label, $table, $rows, @options, @options_ca, $entry,
         $key, $separator, $t);

   if(not defined($name)) {
      $name = $self->{'CA'}->{'actca'};
   }
   if(not defined($name)) {
      $self->print_warning(gettext("Can't get CA name"));
      return;
   }

   $self->{'TCONFIG'}->init_config($self, $name);

   $box = Gnome::PropertyBox->new();
   $box->set_position('none');
   $box->set_policy(0, 0, 0);
   $box->set_modal(0);

   $box->signal_connect('apply', 
         sub { $self->{'TCONFIG'}->write_config($self, $name) });

   $t = gettext("All Settings are written unchanged to openssl.conf.\nSo please study the documentation of OpenSSL if you don't know exactly what to do.\nIf you are still unsure - keep the defaults and everything is expected to work fine.");
   $box->signal_connect('help', 
         sub { $self->print_info($t) });

   $nb = $box->notebook();
   $nb->set_tab_pos('top');
   $nb->set_show_tabs(1);
   $nb->set_show_border(1);
   $nb->set_scrollable(0);


   # first page: vbox with warnings :-)
   $vbox = Gtk::VBox->new(0, 0);

   $label = Gtk::Label->new(gettext("OpenSSL Configuration"));
   $label->set_justify('center');
   $label->set_line_wrap(0);

   $nb->append_page($vbox, $label);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("OpenSSL Configuration"));
   $label->set_style($self->{'stylebold'});
   $label->set_line_wrap(0);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $separator = Gtk::HSeparator->new();
   $vbox->pack_start($separator, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(
         gettext("Only change these options, if you really know, what you are doing!!"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(
         gettext("You should be aware, that some options may brake some crappy software!!"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(
         gettext("If you are unsure: leave the defaults untouched"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   # second page: server settings
   @options = qw(
         nsCertType
         nsComment
         authorityKeyIdentifier
         subjectAltName
         issuerAltName
         crlDistributionPoints
         nsCaRevocationUrl
         nsBaseUrl
         nsRevocationUrl
         nsRenewalUrl
         nsCaPolicyUrl
         );

   @options_ca = qw(
         default_days
         );
   $vbox = Gtk::VBox->new(0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("These Settings are passed to OpenSSL for creating Server Certificates"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("Multiple Values can be separated by \",\""));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $separator = Gtk::HSeparator->new();
   $vbox->pack_start($separator, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $vbox->pack_start($table, 1, 1, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(gettext("Server Certificate Settings"));
   $label->set_justify('center');
   $label->set_line_wrap(0);

   $nb->append_page($vbox, $label);

   foreach $key (@options) { 
      $label = Gtk::Label->new($key);
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'server_cert'}->{$key})
         if(defined($self->{'TCONFIG'}->{'server_cert'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry, 
            \$self->{'TCONFIG'}->{'server_cert'}->{$key}, $box);
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   foreach $key (@options_ca) {
      $label = Gtk::Label->new($key);
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'server_ca'}->{$key})
         if(defined($self->{'TCONFIG'}->{'server_ca'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry, 
            \$self->{'TCONFIG'}->{'server_ca'}->{$key}, $box);
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   # third page: client settings
   @options = qw(
         nsCertType
         nsComment
         authorityKeyIdentifier
         subjectAltName
         issuerAltName
         crlDistributionPoints
         nsCaRevocationUrl
         nsBaseUrl
         nsRevocationUrl
         nsRenewalUrl
         nsCaPolicyUrl
         );


   @options_ca = qw(
         default_days
         );
   $vbox = Gtk::VBox->new(0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("These Settings are passed to OpenSSL for creating Client Certificates"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("Multiple Values can be separated by \",\""));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $separator = Gtk::HSeparator->new();
   $vbox->pack_start($separator, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $vbox->pack_start($table, 1, 1, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(gettext("Client Certificate Settings"));
   $label->set_justify('center');
   $label->set_line_wrap(0);

   $nb->append_page($vbox, $label);

   foreach $key (@options) { 
      $label = Gtk::Label->new($key);
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'client_cert'}->{$key})
         if(defined($self->{'TCONFIG'}->{'client_cert'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry, 
            \$self->{'TCONFIG'}->{'client_cert'}->{$key}, $box);
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   foreach $key (@options_ca) {
      $label = Gtk::Label->new($key);
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'client_ca'}->{$key})
         if(defined($self->{'TCONFIG'}->{'client_ca'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry, 
            \$self->{'TCONFIG'}->{'client_ca'}->{$key}, $box);
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   # fourth page: crl settings
   @options = qw(
         default_crl_days
         );

   $vbox = Gtk::VBox->new(0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("These Settings are passed to OpenSSL for creating Certificate Revocation Lists"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("Multiple Values can be separated by \",\""));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $separator = Gtk::HSeparator->new();
   $vbox->pack_start($separator, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $vbox->pack_start($table, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(gettext("Revocation List Settings"));
   $label->set_justify('center');
   $label->set_line_wrap(0);

   $nb->append_page($vbox, $label);

   foreach $key (@options) { 
      $label = Gtk::Label->new($key);
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'server_ca'}->{$key})
         if(defined($self->{'TCONFIG'}->{'server_ca'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry, 
            \$self->{'TCONFIG'}->{'server_ca'}->{$key}, $box);
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   $box->show_all();

   return;
}

#
# configuration for CA
#
sub show_config_ca {
   my $self = shift;
   my $opts = shift;

   my(@options, $key, $box, $button_ok, $button_cancel, $table, $label,
         $entry, $rows);

   @options = qw(
         authorityKeyIdentifier
         basicConstraints
         keyUsage
         nsCertType
         issuerAltName
         nsComment
         crlDistributionPoints
         nsCaRevocationUrl
         nsCaPolicyUrl
         nsRevocationUrl
         nsPolicyUrl
         );
   
   if(not defined($opts->{'name'})) {
      $self->print_warning(gettext("Can't get CA name"));
      return;
   }

   $self->{'TCONFIG'}->init_config($self, $opts->{'name'});

   $box = Gnome::Dialog->new(gettext("CA Configuration"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { 
            $self->{'TCONFIG'}->write_config($self, $opts->{'name'});
            $opts->{'configured'} = 1;
            $self->{'CA'}->create_ca($self, $opts, $box) });
   $button_cancel->signal_connect('clicked', 
         sub { $box->destroy() });

   $label = Gtk::Label->new(gettext("CA Configuration"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(
         gettext("These Settings are passed to OpenSSL for creating the CA Certificate"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(gettext("Multiple Values can be separated by \",\""));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $label = Gtk::Label->new(
         gettext("If you are unsure: leave the defaults untouched"));
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $rows = 1;
   $table = Gtk::Table->new($rows, 2, 0);
   $box->vbox->add($table);

   
   foreach $key (@options) {
      $label = Gtk::Label->new("$key:");
      $label->set_justify('left');
      $label->set_alignment(0, 0);
      $table->attach_defaults($label, 0, 1, $rows-1, $rows);

      $entry = Gtk::Entry->new();
      $entry->set_text($self->{'TCONFIG'}->{'v3_ca'}->{$key})
         if(defined($self->{'TCONFIG'}->{'v3_ca'}->{$key}));
      $entry->signal_connect('changed', \&_entry_to_var, $entry,
            \$self->{'TCONFIG'}->{'v3_ca'}->{$key});
      $table->attach_defaults($entry, 1, 2, $rows-1, $rows);

      $rows++;
      $table->resize($rows, 2);
   }

   $box->show_all();

   return;
}

#
# ask if the CA shall be converted
#
sub show_ca_convert_dialog {
   my $self = shift;
   my $opts = shift;

   my($box, $label, $button_ok, $button_cancel, $t);

   $box = Gnome::Dialog->new(gettext("CA Configuration"));
   $box->close_hides(0);
   $box->set_close(0);
   $box->set_position('center');
   $box->set_policy(0, 1, 0);
   $box->set_modal(1);
   
   $button_ok     = Gnome::Stock->button('Button_Ok');
   $button_cancel = Gnome::Stock->button('Button_Cancel');

   $box->action_area->set_layout('spread');
   $box->action_area->set_spacing(6 );
   $box->action_area->set_child_ipadding(7, 0 );
   $box->action_area->pack_start( $button_ok, 1, 1, 0 );
   $box->action_area->pack_start( $button_cancel, 1, 1, 0 );

   $button_ok->signal_connect('clicked', 
         sub { 
            $opts->{'doconv'} = 1;
            $self->{'CA'}->open_ca($self, $opts, $box) 
         }
   );
   $button_ok->can_default(1);
   $button_ok->grab_default();
   $button_cancel->signal_connect('clicked', 
         sub { 
            $opts->{'noconv'} = 1;
            $self->{'CA'}->open_ca($self, $opts, $box) 
         }
   );

   $label = Gtk::Label->new(gettext("Convert CA"));
   $label->set_style($self->{'stylebold'});
   $label->set_justify('center');
   $label->set_line_wrap(0);
   $label->set_alignment(0.5, 0.5);
   $box->vbox->add($label);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $t = gettext("This CA seems to be created with openssl 0.9.6x. And it seems like you have switched to openssl 0.9.7x.");

   $label = Gtk::Label->new($t);
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->add($label);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $t = gettext("You won't be able to revoke the existing certificates without converting the index file of this CA to the new format.");

   $label = Gtk::Label->new($t);
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->add($label);
   
   $label = Gtk::Label->new(' ');
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->pack_start($label, 0, 0, 0);

   $t = gettext("Attention: it will not be easy to switch back, this has to be done manually");
   $label = Gtk::Label->new($t);
   $label->set_justify('center');
   $label->set_line_wrap(1);
   $box->vbox->add($label);

   $box->show_all();

   return;
}

# replace pem|der|txt suffix of filename and change status
# parameters:
#	- reference to the variable that holds the filename
#	- reference to variable with new suffix
#	- references to the buttons
sub _changeformat_key($$$$$) {
   my $outfileref  = shift;
   my $formatref   = shift;
   my $passbutton1 = shift;
   my $passbutton2 = shift;

   if($$outfileref =~ s/\.(pem|der|txt|p12|zip)$//i) {
      $$outfileref .= "." . lc $$formatref;
   }
   
   if($$formatref eq "PEM") {
      $passbutton1->configure( -state => "normal");
      $passbutton2->configure( -state => "normal");
   } elsif ($$formatref eq "DER") {
      $passbutton1->configure( -state => "disabled");
      $passbutton2->configure( -state => "disabled");
   } elsif ($$formatref eq "P12") {
      $passbutton1->configure( -state => "disabled");
      $passbutton2->configure( -state => "disabled");
   } elsif ($$formatref eq "ZIP") {
      $passbutton1->configure( -state => "disabled");
      $passbutton2->configure( -state => "disabled");
   }

      
}

#
# called if request is selected
# fill in the request details in the window
#
sub _fill_req_info {
   my ( $clist, $self, $row, $column, $event, @data ) = @_;

   my $ind = $clist->get_text($row, 7);
   my $ca  = $self->{'CA'}->{'actca'};

   my($req, $reqname, $parsed, $label);

   if(not defined($ind)) {
      $self->print_info(gettext("Please select a request first"));
      return;
   }

   $req = $self->{'REQ'}->{'reqlist'}->[$ind];

   $reqname = MIME::Base64::encode($req, '');

   $parsed = $self->{'REQ'}->parse_req($self, $reqname);
   
   defined($parsed) || $self->print_error(gettext("Can't read request file"));

   # hbox in the bottom
   if(defined($self->{'reqbottombox'})) {
      $self->{'reqbottombox'}->destroy();
   }
   $self->{'reqbottombox'} = Gtk::HBox->new(1, 0);
   $self->{'reqtextbox'}->pack_start($self->{'reqbottombox'}, 0, 0, 0);

   # vbox in the bottom/left
   if(defined($self->{'reqleftbox'})) {
      $self->{'reqleftbox'}->destroy();
   }
   $self->{'reqleftbox'} = Gtk::VBox->new(0, 0);
   $self->{'reqbottombox'}->add($self->{'reqleftbox'});

   # vbox in the bottom/right
   if(defined($self->{'reqrightbox'})) {
      $self->{'reqrightbox'}->destroy();
   }
   $self->{'reqrightbox'} = Gtk::VBox->new(0, 0);
   $self->{'reqbottombox'}->add($self->{'reqrightbox'});

   # now fill the left vbox
   if(defined($parsed->{'CN'})){
      $label = Gtk::Label->new(
            gettext("Common Name:")." ".$parsed->{'CN'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'EMAIL'})){
      $label = Gtk::Label->new(
            gettext("eMail Address:")." ".$parsed->{'EMAIL'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'O'})){
      $label = Gtk::Label->new(
            gettext("Organization:")." ".$parsed->{'O'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'OU'})){
      foreach(@{$parsed->{'OU'}}) {
         $label = Gtk::Label->new(
               gettext("Organizational Unit:")." ".$_);
         $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
      }
   }
     
   if(defined($parsed->{'L'})){
      $label = Gtk::Label->new(
            gettext("Location:")." ".$parsed->{'L'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'ST'})){
      $label = Gtk::Label->new(
            gettext("State:")." ".$parsed->{'ST'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'C'})){
      $label = Gtk::Label->new(
            gettext("Country:")." ".$parsed->{'C'});
      $self->{'reqleftbox'}->pack_start($label, 0, 0, 0);
   }

   # now fill the right vbox
   if(defined($parsed->{'KEYSIZE'})){
      $label = Gtk::Label->new(
            gettext("Keylength:")." ".$parsed->{'KEYSIZE'});
      $self->{'reqrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'PK_ALGORITHM'})){
      $label = Gtk::Label->new(
            gettext("Public Key Algorithm:")." ".$parsed->{'PK_ALGORITHM'});
      $self->{'reqrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'SIG_ALGORITHM'})){
      $label = Gtk::Label->new(
            gettext("Signature Algorithm:")." ".$parsed->{'SIG_ALGORITHM'});
      $self->{'reqrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'TYPE'})){
      $label = Gtk::Label->new(
            gettext("Type:")." ".$parsed->{'TYPE'});
      $self->{'reqrightbox'}->pack_start($label, 0, 0, 0);
   }

   $self->{'reqtextbox'}->show_all();
      
   return;
}

#
# create popup menu for keys
#
sub _create_key_menu {
   my $self = shift;

   my ($menu_item);

   $self->{'keymenu'} = Gtk::Menu->new();

   $menu_item = Gnome::Stock->menu_item('Menu_Save',
         gettext("Export Key"));
   $self->{'keymenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'KEY'}->get_export_key($self) });

   $menu_item = Gnome::Stock->menu_item('Menu_Trash',
         gettext("Delete Key"));
   $self->{'keymenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'KEY'}->get_del_key($self) });

   $self->{'keymenu'}->show_all();

   return;
}

#
# called on rightclick in keylist
#
sub _show_key_menu {
   my ($clist, $self, $event) = @_;

   if ((defined($event->{'type'})) &&
         $event->{'button'} == 3) {
      $self->{'keymenu'}->popup( 
            undef,
            undef,
            0,
            $event->{'button'},
            undef);

      return(1);
   }

   return(0);
}

#
# create popup menus for certificates
#
sub _create_cert_menu {
   my $self = shift;

   my ($menu_item);

   $self->{'certmenu'} = Gtk::Menu->new();

   $menu_item = Gnome::Stock->menu_item('Menu_Search', 
         gettext("Certificate Details"));
   $self->{'certmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->show_details('cert') });

   $menu_item = Gnome::Stock->menu_item('Menu_Search', 
         gettext("View Certificate"));
   $self->{'certmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->show_text('cert') });

   $menu_item = Gtk::MenuItem->new();
   $self->{'certmenu'}->append($menu_item);

   $menu_item = Gnome::Stock->menu_item('Menu_Save',
         gettext("Export Certificate"));
   $self->{'certmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'CERT'}->get_export_cert($self) });

   $menu_item = Gnome::Stock->menu_item('Menu_Stop',
         gettext("Revoke Certificate"));
   $self->{'certmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'CERT'}->get_revoke_cert($self) });

   $menu_item = Gnome::Stock->menu_item('Menu_Trash',
         gettext("Delete Certificate"));
   $self->{'certmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'CERT'}->get_del_cert($self) });

   $self->{'certmenu'}->show_all();

   return;
}

#
# called on rightclick in certlist
#
sub _show_cert_menu {
   my ($clist, $self, $event) = @_;

   if ((defined($event->{'type'})) &&
         $event->{'button'} == 3) {
      $self->{'certmenu'}->popup( 
            undef,
            undef,
            0,
            $event->{'button'},
            undef);

      return(1);
   }

   return(0);
}

#
# create popup menus for requests
#
sub _create_req_menu {
   my $self = shift;

   my ($menu_item, $opts);

   $self->{'reqmenu'} = Gtk::Menu->new();

   $menu_item = Gnome::Stock->menu_item('Menu_Search',
         gettext("Request Details"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->show_details('req') });

   $menu_item = Gnome::Stock->menu_item('Menu_Search',
         gettext("View Request"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->show_text('req') });

   $menu_item = Gtk::MenuItem->new();
   $self->{'reqmenu'}->append($menu_item);

   $menu_item = Gnome::Stock->menu_item('Menu_Properties',
         gettext("Sign Request (Server)"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $opts->{'type'} = 'server';
         $self->{'REQ'}->get_sign_req($self, $opts) });

   $menu_item = Gnome::Stock->menu_item('Menu_Properties',
         gettext("Sign Request (Client)"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $opts->{'type'} = 'client';
         $self->{'REQ'}->get_sign_req($self, $opts) });

   $menu_item = Gtk::MenuItem->new();
   $self->{'reqmenu'}->append($menu_item);

   $menu_item = Gnome::Stock->menu_item('Menu_Revert',
         gettext("Import Request"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'REQ'}->get_import_req($self) });

   $menu_item = Gnome::Stock->menu_item('Menu_New',
         gettext("New Request"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'REQ'}->get_req_create($self) });

   $menu_item = Gtk::MenuItem->new();
   $self->{'reqmenu'}->append($menu_item);

   $menu_item = Gnome::Stock->menu_item('Menu_Trash',
         gettext("Delete Request"));
   $self->{'reqmenu'}->append($menu_item);
   $menu_item->signal_connect( 'activate', 
         sub { $self->{'REQ'}->get_del_req($self) });

   $self->{'reqmenu'}->show_all();

   return;
}

#
# called on rightclick in reqlist
#
sub _show_req_menu {
   my ($clist, $self, $event) = @_;

   if ((defined($event->{'type'})) &&
         $event->{'button'} == 3) {
      $self->{'reqmenu'}->popup( 
            undef,
            undef,
            0,
            $event->{'button'},
            undef);

      return(1);
   }

   return(0);
}

#
# called if certificate is selected
# fill in the certificate details in the window
#
sub _fill_cert_info {
   my ($clist, $self, $row, $column, $event, @data) = @_;

   my $ind = $clist->get_text($row, 8);
   my $ca  = $self->{'CA'}->{'actca'};

   my($cert, $status, $certname, $parsed, $label);

   if(not defined($ind)) {
      $self->print_warning(gettext("Please select a certificate first"));
      return;
   }

   ($cert, $status) = split(/\%/, $self->{'CERT'}->{'certlist'}->[$ind]);

   $certname = MIME::Base64::encode($cert, '');

   $parsed = $self->{'CERT'}->parse_cert($self, $certname);

   defined($parsed) || $self->print_error(gettext("Can't read certificate"));

   # fingerprint in the top of certtextbox
   if(defined($self->{'certfingerprint'})) {
      $self->{'certfingerprint'}->destroy();
   } 
   $self->{'certfingerprint'} = Gtk::Label->new(
         gettext("Fingerprint:")." ".$parsed->{'FINGERPRINT'});
   $self->{'certtextbox'}->pack_start($self->{'certfingerprint'}, 0, 0, 0);

   # hbox in the bottom
   if(defined($self->{'certbottombox'})) {
      $self->{'certbottombox'}->destroy();
   }
   $self->{'certbottombox'} = Gtk::HBox->new(1, 0);
   $self->{'certtextbox'}->pack_start($self->{'certbottombox'}, 0, 0, 0);

   # vbox in the bottom/left
   if(defined($self->{'certleftbox'})) {
      $self->{'certleftbox'}->destroy();
   }
   $self->{'certleftbox'} = Gtk::VBox->new(0, 0);
   $self->{'certbottombox'}->add($self->{'certleftbox'});

   # vbox in the bottom/right
   if(defined($self->{'certrightbox'})) {
      $self->{'certrightbox'}->destroy();
   }
   $self->{'certrightbox'} = Gtk::VBox->new(0, 0);
   $self->{'certbottombox'}->add($self->{'certrightbox'});

   # now fill the left vbox
   if(defined($parsed->{'CN'})){
      $label = Gtk::Label->new(
            gettext("Common Name:")." ".$parsed->{'CN'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'EMAIL'})){
      $label = Gtk::Label->new(
            gettext("eMail Address:")." ".$parsed->{'EMAIL'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'O'})){
      $label = Gtk::Label->new(
            gettext("Organization:")." ".$parsed->{'O'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'OU'})){
      foreach(@{$parsed->{'OU'}}) {
         $label = Gtk::Label->new(
               gettext("Organizational Unit:")." ".$_);
         $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
      }
   }
     
   if(defined($parsed->{'L'})){
      $label = Gtk::Label->new(
            gettext("Location:")." ".$parsed->{'L'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'ST'})){
      $label = Gtk::Label->new(
            gettext("State:")." ".$parsed->{'ST'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'C'})){
      $label = Gtk::Label->new(
            gettext("Country:")." ".$parsed->{'C'});
      $self->{'certleftbox'}->pack_start($label, 0, 0, 0);
   }

   # now fill the right vbox
   $label = Gtk::Label->new(
         gettext("Status:")." ".$status);
   $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
     
   if(defined($parsed->{'NOTBEFORE'})){
      $label = Gtk::Label->new(
            gettext("Creation Date:")." ".$parsed->{'NOTBEFORE'});
      $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'NOTAFTER'})){
      $label = Gtk::Label->new(
            gettext("Expiration Date:")." ".$parsed->{'NOTAFTER'});
      $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'KEYSIZE'})){
      $label = Gtk::Label->new(
            gettext("Keylength:")." ".$parsed->{'KEYSIZE'});
      $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'PK_ALGORITHM'})){
      $label = Gtk::Label->new(
            gettext("Public Key Algorithm:")." ".$parsed->{'PK_ALGORITHM'});
      $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
   }
     
   if(defined($parsed->{'SIG_ALGORITHM'})){
      $label = Gtk::Label->new(
            gettext("Signature Algorithm:")." ".$parsed->{'SIG_ALGORITHM'});
      $self->{'certrightbox'}->pack_start($label, 0, 0, 0);
   }

   $self->{'certtextbox'}->show_all();
      
   return;
}

#
# fill given var-reference with text from entry
#
sub _entry_to_var {
   my ($widget, $entry, $var, $box) = @_;

   $$var = $entry->get_text();

   $box->set_modified(1) if(defined($box));

   return;
}

#
# fill given var-reference with value from togglebutton
#
sub _toggle_to_var {
   my ($button, $var, $value, $outfileref, $formatref, $fileentry) = @_;

   $$var = $value if($button->active());

   if(defined($outfileref) && defined($formatref)) {
      if($$outfileref =~ s/\.(pem|der|txt|p12|zip)$//i) {
         $$outfileref .= "." . lc $$formatref;
         $fileentry->gnome_entry->entry->set_text($$outfileref);
      }
   }

   return;
}

#
# finished...
#
sub _exit_clean {
   my $self = shift;
   my $ret  = shift;

   $ret = 0 unless(defined $ret);
   
   Gtk->exit($ret);

   exit($ret);
}

1

# 
# $Log: GUI.pm,v $
# Revision 1.7  2003/08/17 17:26:19  sm
# added contect menus in lists
#
# Revision 1.5  2003/08/16 22:05:24  sm
# first release with Gtk-Perl
#
# Revision 1.3  2003/08/13 20:38:51  sm
# functionality done
#
# Revision 1.2  2003/08/13 19:39:36  sm
# rewrite for Gtk
#
# Revision 1.20  2003/07/04 23:21:25  sm
# changed version
#
# Revision 1.19  2003/07/04 22:58:58  sm
# first round of the translation is done
#
# Revision 1.18  2003/07/03 20:59:03  sm
# a lot of gettext() inserted
#
# Revision 1.17  2003/07/03 07:30:01  sm
# inserted a lot of gettext()
#
# Revision 1.16  2003/06/30 22:35:30  sm
# changed version
#
# Revision 1.15  2003/06/30 22:33:18  sm
# added conversion of index.txt
#
# Revision 1.14  2003/06/26 23:28:35  sm
# added zip functions
#
# Revision 1.13  2003/06/26 20:44:31  sm
# added zip patch from ludwig.nussel@suse.de
#
# Revision 1.12  2003/06/23 21:31:17  sm
# changed version
#
# Revision 1.11  2003/06/23 21:16:29  sm
# automatically change filename and buttonstatus during export
#
# Revision 1.10  2003/06/23 20:11:29  sm
# some new texts from ludwig.nussel@suse.de
#
# Revision 1.9  2003/06/19 21:46:43  sm
# change button status dynamically
#
# Revision 1.7  2003/06/19 13:52:03  sm
# made default_crl_days configurable and some more usability stuff
#
# Revision 1.4  2002/10/07 17:33:58  sm
# added horizontal scrollbar to listings
# modified some dialogs (open, delete,...)
#
# Revision 1.3  2002/10/04 15:15:46  sm
# increased version
#
# Revision 1.2  2002/09/27 19:54:50  sm
# Increased version
#
# 
